/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifndef __EXCEPTIONS_H__
#define __EXCEPTIONS_H__

#include <string>
#include <vector>

#define _Exception(msg) dSA::Exception(msg, 0, __FILE__, __LINE__, __func__)
#define _ExceptionWithCode(msg, code) dSA::Exception(msg, code, __FILE__, __LINE__, __func__)

namespace dSA
{

class Exception
{
public:
    Exception(std::string message, int err, const char* file, int line,
              const char* function);
    Exception(std::string message);
    const std::string& getMessage() const;
    int getErrorCode() const;
    const std::vector<std::string> getStackTrace() const;
    void printStackTrace();

protected:
    std::string m_message;
    int m_error_code;
    std::string m_file;
    std::string m_function;
    int m_line;
    std::vector<std::string> m_stackTrace;
};

}

#endif//__EXCEPTIONS_H__
